/*
 * Copyright 2000 by Jean-Claude Christophe <jch@oleane.net>
 *                                          <jch@bsdfr.org>
 *
 * Home page:  http://www.bsdfr.org/projects/tac_bsd/
 *
 * main.c  Core program.
 */

#include <sys/types.h>
#include <sys/socket.h> /* shutdown(2) */
#include <sys/param.h>  /* MAXHOSTNAMELEN in main() */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <strings.h>

#include "draft2.h"
#include "tac_bsd.h"
#include "conf.h"

#define TAC_BSD_VERSION "1.3.3"

/* in login.c */
extern void login_in();
extern int  local_user(void);
extern int  login_local(void);
extern int  login_remote(void);

/* in misc.c */
extern void authenticate(void);

char *username; /* username we will use for authentication */
char *password; /* idem for password */
char *port;     /* current tty */
char *term;     /* our teminal info */
char *hostname; /* the hostname which the user come from */

int try = 0, done = 0; /* some check for the main loop */

void
display_file(file)
    char *file;
{
    FILE *fp;
    char c[256];
    
    if (fp = fopen(file, "r")) {
	while (fgets(c, 256, fp))
	    printf("%s", c);
    }
}

void
close_quit()
{
    int desc;

    for (desc = getdtablesize(); desc > 2; desc--) {
	shutdown(desc, SHUT_RDWR);
	close(desc);
    }
    exit(-1);
}

int
main(argc, argv)
    int argc;
    char *argv[];
{
    char *domain, *p;
    int ch;        /* for getopt(3) */
    char localhost[MAXHOSTNAMELEN];

    /*
     * Before attempt anything, check if the station accepts login
     */
    {
	FILE *fp;
	char c[256];

	if ((fp = fopen(NOLOGIN_FILE, "r"))) {
	    while (fgets(c, 256, fp))
		fprintf(stdout, "%s", c);
	    close_quit();
	}
    }

    domain = NULL;
    if (gethostname(localhost, sizeof(localhost)) >= 0)
	domain = strchr(localhost, '.');
    if (domain) {
	domain++;
	if (*domain && strchr(domain, '.') == NULL)
	    domain = localhost;
    }

    while ((ch = getopt(argc, argv, "h:p")) != -1)
	switch (ch) {
	case 'h':
	    if (domain && (p = strchr(optarg, '.')) &&
		strcasecmp(p + 1, domain) == 0)
		*p = 0;
	    hostname = optarg;
	    break;
	case 'p':
	    break;
	case '?':
	default:
	    break;
	}
    /*
     * not needed here
     *    argc -= optind;
     *    argv += optind;
     */

    if (read_conf(CONF_FILE) < 0) {
	perror(CONF_FILE);
	exit(-1);
    }
    port = ttyname(STDIN_FILENO); /* What is the current tty ? it will be our port.
				   * STDIN_FILENO = 0
				   */
    
    username = (char *)malloc(256);
    if (!username) { /* not enough memory */
	perror("username");
	exit(-1);
    }
    
    /* display motd if set */
    if (premotd) display_file(premotd);
    
    printf("User Access Verification\n\n"); /* cisco's style banner */

    while ((try != maxtry) && !done) {

	/* 
	 * now, it's time to be serious.
	 * Our authentication can be resumed into 4 steps (we use TYPE_ASCII):
	 * 1) after established the connection, we send the START request
	 * 2) we are waiting for the REPLY
	 * 3) If the REPLY seems to be correct, we send the CONTINUE packet
	 * 4) The daemon reply by PASS or FAIL: the result we want.
	 */

	/* ask for username and password */
	authenticate(); /* in misc.c */

	/* We take a look at the local password database only if passwd_care is set */
	if ((local_user() > 0) && passwd_care) {
	    if (login_local() < 0) {
		if (slog)
		    syslog(_syslog, "LOCAL Username %s rejected on %s", username, port);
	    } else
		if (slog)
		    syslog(_syslog, "LOCAL Username %s accepted after %d tries on %s", \
			   username, try + 1, port);
	} else
	    if (login_remote() < 0) {
		if (slog)
		    syslog(_syslog, "TACACS Username %s rejected on %s", username, port);
	    } else
		if (slog)
		    syslog(_syslog, "TACACS Username %s accepted after %d tries on %s", \
			   username, try + 1, port);
 
    } /* while(.. */

    if (try == maxtry) {
	if (motdfail)
	    display_file(motdfail);
	else
	    close_quit();
    }
    
    login_in();

    return(0);
}
